"use strict";
exports.__esModule = true;
var AppModel_1 = require("./model/AppModel");
var OsTools_1 = require("./OsTools");
var fs = require("fs");
var http = require("http");
var AppHandler = /** @class */ (function () {
    /**
     *
     */
    function AppHandler() {
        this.Os = new OsTools_1.OsTools();
        this.loadConfig();
    }
    AppHandler.prototype.log = function (msg) {
        console.log(new Date().toLocaleTimeString() + " " + msg);
    };
    AppHandler.prototype.verbose = function (msg) {
        if (!this.Config || this.Config.Debug) {
            this.log(msg);
        }
    };
    AppHandler.prototype.logError = function (msg) {
        console.log("ERROR " + new Date().toLocaleTimeString() + " " + msg);
    };
    AppHandler.prototype.loadConfig = function () {
        // let c:DikasAppConfig;
        var s = this.Os.configFile;
        this.verbose("Load Config " + s);
        // this.Config = new DikasAppConfig();
        if (fs.existsSync(s)) {
            try {
                this.Config = JSON.parse(fs.readFileSync(s).toString());
            }
            catch (ex) {
                this.logError("Load config Error " + ex);
            }
        }
        // Auf jeden Fall die Config initalisieren
        if (!this.Config) {
            this.verbose("Save new Config in " + s);
            this.Config = new AppModel_1.DikasAppConfig();
            this.saveConfig();
        }
        if (process.argv.length > 0) {
            for (var i = 0, im = process.argv.length; i < im; i++) {
                if (process.argv[i] && process.argv[i][0] === "d") {
                    this.Config.Debug = true;
                }
                else {
                    this.log("args  " + process.argv.join(" , "));
                }
            }
            this.verbose("Debug Mode " + this.Config.Debug);
        }
    };
    AppHandler.prototype.saveConfig = function () {
        try {
            var s = this.Os.configFile;
            this.log("Save config " + s);
            fs.writeFileSync(s, JSON.stringify(this.Config));
        }
        catch (e) {
            this.logError("Save Config Error" + e);
        }
    };
    AppHandler.prototype.doJson = function (url, cb) {
        var _this = this;
        this.doCurl(url, function (db) {
            try {
                cb(JSON.parse(db));
            }
            catch (e) {
                _this.logError("Parse Error " + db);
            }
        });
    };
    AppHandler.prototype.doCurl = function (url, cb) {
        var _this = this;
        try {
            // log("docurl " + url);
            var callback = function (response) {
                var str = "";
                response.on("data", function (chunk) {
                    str += chunk;
                });
                response.on("error", function () {
                    if (cb) {
                        cb(null);
                        cb = null;
                    }
                });
                //the whole response has been recieved, so we just print it out here
                response.on("end", function () {
                    if (cb) {
                        cb(str);
                    }
                });
            };
            http.get(url, callback).on("error", function (e) {
                _this.log("curlerror" + e);
                cb(null);
            }).end();
        }
        catch (e) {
            this.log("curlerror" + e);
        }
    };
    return AppHandler;
}());
exports.AppHandler = AppHandler;
