"use strict";
exports.__esModule = true;
var SerialPort = require("serialport");
var NfcCardReader = /** @class */ (function () {
    /** app ist express app  */
    function NfcCardReader(appHandler) {
        var _this = this;
        this.appHandler = appHandler;
        /**  Handler auf den Comport */
        this.Name = "NfcReader";
        this.WaitList = [];
        this.PortInUse = false;
        this.Key = "";
        this.IsKeyInit = false;
        this.CardValue = "";
        // Default Implementierung des Codes.
        this.waitList = [];
        this.CardKey = [0x7c, 0x31, 0x4e, 0xba, 0x4d, 0x88];
        this.DebugReadWrite = false;
        // Default Com Port
        // let Comport = "/dev/ttyUSB0";
        this.cmd_setFilename = [0x0C, 0x00, 0x09, 0x08, 0x00, 0x03, 0x10, 0x00, 0x04, 0x70, 0x77, 0x31, 0x36];
        this.cmd_MIFMode = [0x0C, 0x00, 0x09, 0x05, 0x00, 0x03, 0x10, 0x10, 0x01, 0x00];
        this.cmd_SETblockList = [0x0C, 0x00, 0x09, 0x33, 0x00, 0x03, 0x10, 0x11, 0x2F, 0x01, 0x02, 0x04, 0x05, 0x06,
            0x08, 0x09, 0x0A, 0x0C, 0x0D, 0x0E, 0x10, 0x11, 0x12, 0x14, 0x15, 0x16, 0x18, 0x19, 0x1A, 0x1C, 0x1D, 0x1E, 0x20,
            0x21, 0x22, 0x24, 0x25, 0x26, 0x28, 0x29, 0x2A, 0x2C, 0x2D, 0x2E, 0x30, 0x31, 0x32, 0x34, 0x35, 0x36, 0x38, 0x39,
            0x3A, 0x3C, 0x3D, 0x3E];
        //                   0     1    2     4     5     6     7     8    9     10
        this.cmd_KeyList = [0x0C, 0x00, 0x09, 0x0A, 0x00, 0x03, 0x10, 0x94, 0x06, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff];
        this.cmd_SetRDKey = [0x0C, 0x00, 0x09, 0x33, 0x00, 0x03, 0x10, 0x12, 0x2F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];
        this.cmd_SetWDKey = [0x0C, 0x00, 0x09, 0x33, 0x00, 0x03, 0x10, 0x13, 0x2F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];
        // 0C-00-09-0A-00-03-10-94-06-7C-31-4E-BA-4D-88
        // let cmd_setupmifarekey = [0x0C, 0x01, 0x08, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF]
        this.cmd_getpuffer = [0x0C, 0x00, 0x00, 0x00, 0x00];
        this.cmd_searchCard = [0x0C, 0x01, 0x00, 0x04, 0x00, 0xFF, 0xFF, 0x01, 0x00];
        this.cmd_getCardSerial = [0x0c, 0x01, 0x01, 0x00, 0x00];
        // Wir lesen die ersten 4 bytes aus
        this.cmd_readHeader = [0x0C, 0x01, 0x02, 0x05, 0x00, 0x10, 0x00, 0x00, 0x00, 0x04];
        // Wir schreiben 32 bytes den kompletten karten kopf
        this.cmd_writeHeader = [0x8C, 0x01, 0x03, 0x25, 0x00, 0x10, 0x00, 0x00, 0x00, 0x20];
        this.lastCardSerial = [0, 0, 0, 0];
        this.currentSerial = 0;
        this.sameCardIntervall = null;
        this.maxPortinUseTime = 3;
        this.portResetErrors = 0;
        this.isFirstRun = false;
        var imaxCheck = 5000 / 250;
        this.isNewMode = false;
        this.checkCardTimer = setInterval(function () {
            // lesen der Karte
            _this.checkCardOnReader();
            if (imaxCheck-- < 0) {
                imaxCheck = 5000 / 250;
                // Ok nuen Initalliseren
                _this.startSerial();
                _this.clearWaitList();
            }
        }, 250);
        var comport = null;
        // Wenn wir einen Comport in der Config haben nehmen wir den her.
        if (appHandler.Config.NfcPort && appHandler.Config.NfcPort !== "auto") {
            var p = parseInt(appHandler.Config.NfcPort);
            // Je nach OS
            if (appHandler.Os.isLinux) {
                // Ok damit wir den ranbekommen
                if (p > -1) {
                    // 0-9 sind die Comports
                    comport = "/dev/ttyS" + p;
                }
                else {
                    // -1 -n sind die USB
                    // Umdrehen und 1 abziehen
                    p = (p * -1) - 1;
                    comport = "/dev/ttyUSB" + p;
                }
            }
            else {
                if (p > 0) {
                    comport = "com" + (p + 1);
                }
                else {
                    this.appHandler.log(" NFC:Windows use Com1");
                    comport = "com1";
                }
            }
            // So geben wir einfach mal so aus
            this.appHandler.log("NFC: Start mit Port " + comport);
        }
        else {
            this.appHandler.log("NFC: Starte Auto Comport");
        }
        this.Comport = comport;
        // Suchen nach dem Karten tsclesern
        this.startSerial();
        this.initRestUrls(this.appHandler.App);
    }
    /** Beendet den Kartenleser ggf wenn sich der Port geändert hat */
    NfcCardReader.prototype.doDestroy = function () {
        clearInterval(this.checkCardTimer);
        this.closeSerialPort();
        this.serialPort = null;
    };
    NfcCardReader.prototype.haveReader = function (cb) {
        if (this.serialPort) {
            if (cb) {
                cb(true);
            }
        }
        else {
            // Ok dann prüfen wir das so ab.
            this.checkReaderExists(cb);
        }
    };
    NfcCardReader.prototype.logError = function (e) {
        this.appHandler.logError(e + "\n" + e.stack);
    };
    NfcCardReader.prototype.initRestUrls = function (app) {
        var _this = this;
        app.get("/rest/nfc/setkey/:key", function (req, res) {
            var key = req.params.key;
            if (key.length === 12) {
                // neusetzen des schlüssels nur wenn wir das brauchen
                if (key !== _this.Key) {
                    _this.Key = key;
                    _this.IsKeyInit = false;
                }
            }
        });
        // Ok wir binden uns nun einfach in den App Stream ein :-)
        app.get("/rest/nfc", function (req, res) {
            // Ok wenn wir neuen Modus bekommen 
            _this.isNewMode = req.query["mode"] == "1";
            // console.log("New Mode " + this.isNewMode);
            if (!_this.serialPort) {
                _this.startSerial(function (isReader) {
                    _this.handleCardRequest(req, res);
                });
            }
            else {
                _this.handleCardRequest(req, res);
            }
        });
        // app.get("/rest/nfc/write/:cardid/:newdata", (req, res) => {
        //     let newData = req.params.newdata;
        //     let cardid = req.params.cardid;
        //     this.writeToCard(cardid, newData, (data) => {
        //         res.json(data);
        //         res.end();
        //     })
        // });
        app.get("/rest/nfcset/:id", function (req, res) {
            if (this.IsSimulateCardReader) {
                this.currentSerial = req.params.id;
            }
            this.sendResponse(res);
        });
    };
    NfcCardReader.prototype.handleCardRequest = function (req, res) {
        if (this.serialPort) {
            if (!req.query.id || (req.query.id && (this.currentSerial === req.query.id))
                || !this.currentSerial) {
                this.appHandler.log("Add to Watchlist " + this.Name);
                if (req.query.key) {
                    if (this.Key !== req.query.key) {
                        this.Key = req.query.key;
                        this.IsKeyInit = false;
                    }
                }
                this.WaitList.push({ request: res, date: new Date().getTime() });
            }
            else {
                this.sendResponse(res, this.currentSerial);
                // this.currentSerial = 0;
            }
        }
        else {
            setTimeout(function () {
                res.sendStatus(404);
            }, 2000);
        }
    };
    NfcCardReader.prototype.closeSerialPort = function () {
        this.appHandler.log("Cardreader close Port " + this.serialPort);
        if (this.serialPort) {
            try {
                this.PortInUse = false;
                this.serialPort.close();
            }
            catch (e) {
                this.logError(e);
            }
            this.serialPort = null;
        }
    };
    NfcCardReader.prototype.handleNewCard = function (serial) {
        var _this = this;
        this.currentSerialKey = serial;
        // Die Karte für eine halbe Sekunde als aktuelle anzeigen
        setTimeout(function () {
            _this.appHandler.log("Reset Card to 0");
            _this.currentSerial = 0;
        }, 300);
        var newserial = this.byteArrayToLong(serial);
        this.currentSerial = newserial;
        if (this.WaitList) {
            // Nun Arbeiten wir die offenen Request ab danach sind die eh beendet :-)
            while (this.WaitList.length > 0) {
                this.appHandler.log("Send CardInfo to " + this.WaitList.length + " " + newserial);
                var res = this.WaitList.shift();
                try {
                    // Kann ja schon beendet sein
                    this.sendResponse(res.request, newserial);
                }
                catch (e) {
                    this.logError(e);
                }
            }
        }
    };
    NfcCardReader.prototype.backcheckCard = function (data) {
        if (data && data.length > 0) {
            //  0    1    2    3     4    5
            // 0x8C,0x01,0x00,0x01,0x00,0x01
            // 0x0C,0x01,0x00,0x01,0x00,0x10
            // 0x0C,0x01,0x00,0x01,0x00,0x10
            // ERR_NOTAG = 0x01,
            if (data[5] > 1) {
                this.handleCardFound();
                return;
            }
        }
        if (this.currentSerial) {
            this.currentSerial = 0;
            this.lastCardSerial = [0, 0, 0, 0];
            this.handleNewCard(this.lastCardSerial);
        }
    };
    NfcCardReader.prototype.checkCardOnReader = function () {
        if (this.WaitList && this.WaitList.length > 0) {
            if (this.serialPort) {
                if (this.PortInUse) {
                    this.maxPortinUseTime--;
                    if (this.maxPortinUseTime < 0) {
                        this.portResetErrors++;
                        this.appHandler.log("PortInUse Reset " + this.portResetErrors);
                        // Dem System 2 Sekunden geben bis wir den in Use Resetten
                        this.maxPortinUseTime = 8;
                        this.PortInUse = false;
                        if (this.portResetErrors > 2) {
                            this.closeSerialPort();
                            this.portResetErrors = 0;
                        }
                    }
                }
                else {
                    this.PortInUse = true;
                    this.serialCallBackHandler = this.backcheckCard;
                    this.serialPort.write(this.cmd_searchCard);
                }
            }
            else {
                this.startSerial();
            }
        }
        else {
            this.lastCardSerial.length = 0;
        }
    };
    NfcCardReader.prototype.checkReaderExists = function (cb) {
        var _this = this;
        if (!this.Comport) {
            if (this.lastCheckDate && this.lastCheckDate > new Date()) {
                cb(false);
            }
            else {
                // nur jede Minute mal testen lassen
                this.lastCheckDate = new Date(new Date().getTime() + 6000);
                SerialPort.list(function (err, ports) {
                    if (ports) {
                        if (!_this.isFirstRun) {
                            _this.isFirstRun = true;
                            ports.forEach(function (port) {
                                // nur ports anzeigen die wir auch wirklich  gefunden haben. 
                                if (port.manufacturer) {
                                    _this.appHandler.log("found Comport: " + port.comName + " " + port.manufacturer);
                                }
                            });
                        }
                        var target = ports.find(function (port) {
                            // this.appHandler.log("Comport : " + port.manufacturer + " " + isFound);
                            if (port.manufacturer) {
                                if (port.manufacturer.toLocaleLowerCase().indexOf("baltech") > -1
                                    || port.manufacturer.toLocaleLowerCase().indexOf("silicon_labs") > -1) {
                                    _this.appHandler.log("NFC on " + port.comName + " " + port.manufacturer);
                                    return true;
                                }
                            }
                            return false;
                        });
                        if (target) {
                            _this.Comport = target.comName;
                        }
                    }
                });
                return;
            }
        }
        else {
            if (cb) {
                cb(true);
            }
        }
    };
    NfcCardReader.prototype.sendResponse = function (res, cardid, changed) {
        try {
            if (res) {
                // res.writeHead(200, { "Content-Type": "application/json" ,"Access-Control-Allow-Origin":"*"});
                var hand = { Id: cardid, Key: this.currentSerialKey, Card: this.CardValue, C: null };
                if (changed) {
                    hand.C = changed;
                }
                res.json(hand);
            }
        }
        catch (e) {
            this.logError(e);
        }
    };
    NfcCardReader.prototype.backFoundCard = function (data) {
        var target = [0];
        if (data) {
            // Wir kopieren den Puffer erst mal in unseren Array
            for (var i = 0, im = data.length; i < im; i++) {
                this.recBuffer.push(data[i]);
            }
            // Ja nach Modus müssen wir 4 byte oder 7 Byte auslesen
            if (this.recBuffer[3] === 0x04 && this.recBuffer.length > 8) {
                // targetData.Serial = [data[5], data[6], data[7], data[8]];
                // Da wir von Android die auch so bekommen.
                target = [this.recBuffer[8], this.recBuffer[7], this.recBuffer[6], this.recBuffer[5]];
            }
            else if (this.recBuffer[3] === 0x07 && this.recBuffer.length > 11) {
                target = [this.recBuffer[11], this.recBuffer[10], this.recBuffer[9], this.recBuffer[8],
                    this.recBuffer[7], this.recBuffer[6], this.recBuffer[5]];
            }
            else {
                // Sicherstellen das wir warten bis wir alles glesen haben
                console.log("Wait for Bytes " + this.recBuffer.length + " " + this.recBuffer[3]);
                return;
            }
            if (this.createHexString(this.lastCardSerial) !== this.createHexString(target)) {
                this.lastCardSerial = target;
                this.appHandler.log("Card " + this.createHexString(target));
                this.CardValue = "";
                this.handleNewCard(this.lastCardSerial);
                // if (this.Key && this.Key.length > 0) {
                //     if (!this.IsKeyInit) {
                //         // TODO: DEN Key den wir bekommen haben noch in die Key Maske patchen
                //         // Muss hier passieren
                //         let keyData = this.parseHexString(this.Key);
                //         if (keyData.length == 6) {
                //             for (let i = 0; i < keyData.length; i++) {
                //                 this.cmd_KeyList[9 + i] = keyData[i];
                //             }
                //             this.appHandler.log("SendKey " + this.createHexString(this.cmd_KeyList));
                //         }
                //         this.initMifareKeys(() => {
                //             this.IsKeyInit = true;
                //             this.readCard();
                //         }, 0)
                //     } else {
                //         this.readCard();
                //     }
                // } else {
                //     this.handleNewCard(this.lastCardSerial);
                // }
            }
        }
    };
    NfcCardReader.prototype.initMifareKeys = function (cb, callerid) {
        var _this = this;
        if (!callerid) {
            callerid = 0;
        }
        this.appHandler.log("Init Mifare Step " + callerid);
        this.PortInUse = true;
        var cmd = [];
        switch (callerid) {
            case 0:
                cmd = this.cmd_setFilename;
                break;
            case 1:
                cmd = this.cmd_MIFMode;
                break;
            case 2:
                cmd = this.cmd_SETblockList;
                break;
            case 3:
                cmd = this.cmd_KeyList;
                break;
            case 4:
                cmd = this.cmd_SetRDKey;
                break;
            case 5:
                cmd = this.cmd_SetWDKey;
                break;
            default:
                // OK machen wir nix
                break;
        }
        // So nun senden wir solange wir wir alle durch haben
        this.serialCallBackHandler = function (data) {
            _this.appHandler.log("Init result " + _this.createHexString(data));
            if (callerid < 5) {
                _this.initMifareKeys(cb, callerid + 1);
            }
            else {
                _this.PortInUse = false;
                cb();
            }
        };
        if (this.serialPort) {
            this.serialPort.write(cmd);
        }
    };
    // private writeToCard(carid: number, data, cb, maxcount: number = 1) {
    //     if (carid == this.currentSerial) {
    //         if (data.length > 2) {
    //             this.PortInUse = true;
    //             // Wird ausgeführt wenn wir zurück kommen
    //             this.serialCallBackHandler = (resdata) => {
    //                 this.PortInUse = false;
    //                 this.readCardCb((rdata) => {
    //                     if (rdata == data) {
    //                         this.appHandler.log("Write Card: OK " + rdata);
    //                         cb({ IsOk: true });
    //                     } else {
    //                         if (maxcount > 3) {
    //                             this.appHandler.log("Write Card: NOK " + rdata + " to" + data);
    //                             cb({ IsOk: false, Message: "Cardwrite error" });
    //                         } else {
    //                             this.appHandler.log("Write Card: NOK " + rdata + " soll " + data + " versuch " + maxcount);
    //                             if (this.serialPort) {
    //                                 this.writeToCard(carid, data, cb, maxcount + 1);
    //                             }
    //                         }
    //                     }
    //                 });
    //             }
    //             try {
    //                 let newArray = [0x8C, 0x01, 0x03, 0x25, 0x00, 0x10, 0x00, 0x00, 0x00, 0x20, 0xDD,
    // 0xCC, 0xAA, 0xBB, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    // 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00];
    //                 this.appHandler.log("Write " + data);
    //                 let newArray2 = this.parseHexString(data);
    //                 for (let i = 0, im = newArray2.length; i < im; i++) {
    //                     newArray[10 + i] = newArray2[i];
    //                 }
    //                 this.appHandler.log("Send " + this.createHexString(newArray));
    //                 if (this.serialPort) {
    //                     this.serialPort.write(newArray);
    //                 }
    //             } catch (e) {
    //                 this.appHandler.log("SerialPort:" + e);
    //                 this.closeSerialPort();
    //             }
    //         } else {
    //             this.appHandler.log("Write Card DATA zu lange " + data.length);
    //         }
    //     } else {
    //         cb({ IsOk: false, Message: "falsche Karte auf dem Leser" });
    //     }
    // }
    // private readCard() {
    //     // app.log("Read Card");
    //     this.CardValue = "";
    //     // Wir haben die Karte auf dem Leser nun suchen wir nach dem Kopf der Karte.
    //     // Und tragen ein was passiert wenn wir den Kopf gefunden haben
    //     this.PortInUse = true;
    //     this.serialCallBackHandler = this.readCardBack
    //     try {
    //         if (this.serialPort) {
    //             this.serialPort.write(this.cmd_readHeader);
    //         }
    //     } catch (e) {
    //         this.appHandler.log("SerialPort:" + e);
    //         this.closeSerialPort();
    //     }
    // }
    // private readCardCb(cb) {
    //     // app.log("Read Card");
    //     // this.srv.CardValue = "";
    //     // Wir haben die Karte auf dem Leser nun suchen wir nach dem Kopf der Karte.
    //     // Und tragen ein was passiert wenn wir den Kopf gefunden haben
    //     this.PortInUse = true;
    //     this.serialCallBackHandler = (data) => {
    //         this.PortInUse = false;
    //         let dataSteam = this.createHexString(data);
    //         if (dataSteam.length == 12) {
    //             dataSteam = dataSteam.substr(4);
    //         } else if (dataSteam.length == 18) {
    //             dataSteam = dataSteam.substr(10);
    //         } else {
    //             this.appHandler.log("Card Error:" + dataSteam);
    //         }
    //         this.appHandler.log("Card Data:" + dataSteam);
    //         cb(dataSteam);
    //     }
    //     try {
    //         this.serialPort.write(this.cmd_readHeader);
    //     } catch (e) {
    //         this.appHandler.log("SerialPort Error" + e);
    //         this.closeSerialPort();
    //     }
    // }
    // private readCardBack(data) {
    //     this.PortInUse = false;
    //     let dataSteam = this.createHexString(data);
    //     if (dataSteam.length == 12) {
    //         dataSteam = dataSteam.substr(4);
    //     } else if (dataSteam.length == 18) {
    //         dataSteam = dataSteam.substr(10);
    //     } else {
    //         this.appHandler.log("Card Error:" + dataSteam);
    //         this.handleNewCard(this.lastCardSerial);
    //         return;
    //     }
    //     this.appHandler.log("Card Data:" + dataSteam);
    //     this.CardValue = dataSteam;
    //     this.handleNewCard(this.lastCardSerial);
    // }
    NfcCardReader.prototype.handleCardFound = function () {
        if (!this.PortInUse) {
            this.PortInUse = true;
            // Sicherstellen das wir den Puffer haben
            this.recBuffer = [];
            // Wir haben die Karte auf dem Leser nun suchen wir nach dem Kopf der Karte.
            // Und tragen ein was passiert wenn wir den Kopf gefunden haben
            this.serialCallBackHandler = this.backFoundCard;
            if (this.serialPort) {
                this.serialPort.write(this.cmd_getCardSerial);
            }
        }
    };
    NfcCardReader.prototype.startSerial = function (cb) {
        var _this = this;
        // Empty Callback
        if (!cb) {
            cb = function () {
                // Empty Handler
            };
        }
        if (!this.serialPort) {
            this.checkReaderExists(function (res) {
                if (res) {
                    // OK wir sind an der Arbeit
                    _this.serialPort = {};
                    _this.appHandler.log("Open Comport " + _this.Comport);
                    // Sicherstellen das wir auch den port in benutzung haben
                    _this.PortInUse = true;
                    // Alter und neuer Style
                    if (SerialPort.SerialPort) {
                        // Ok nun machen wir den Port auf
                        _this.serialPort = new SerialPort.SerialPort(_this.Comport, {
                            baudrate: 115200, databits: 8,
                            stopbits: 1, parity: "none", parser: SerialPort.parsers.raw
                        });
                    }
                    else {
                        _this.serialPort = new SerialPort(_this.Comport, {
                            baudRate: 115200,
                            dataBits: 8, stopBits: 1, parity: "none", parser: SerialPort.parsers.raw
                        });
                    }
                    _this.serialPort.on("close", function () {
                        _this.closeSerialPort();
                    });
                    // Wenn wir was zurück bekommen.
                    _this.serialPort.on("data", function (data) {
                        _this.PortInUse = false;
                        if (_this.DebugReadWrite) {
                            _this.appHandler.log("data: " + _this.createHexString(data));
                        }
                        if (_this.serialCallBackHandler != null) {
                            _this.serialCallBackHandler(data);
                        }
                    });
                    _this.serialPort.on("error", function (msg) {
                        // Brauchen wir sonst ja nicht.
                        if (msg && msg.toString().indexOf("Port is opening") === -1) {
                            _this.appHandler.log(msg);
                        }
                    });
                    _this.serialPort.open(function () {
                        _this.appHandler.log("Cardreader open " + _this.Comport);
                        _this.PortInUse = true;
                        _this.serialCallBackHandler = function () {
                            _this.appHandler.log("Cardreader Init");
                            _this.PortInUse = false;
                        };
                        _this.serialPort.write(_this.cmd_getpuffer);
                        if (cb) {
                            cb(true);
                        }
                    });
                }
                else {
                    cb(false);
                }
            });
        }
    };
    NfcCardReader.prototype.parseHexString = function (str) {
        var a = [];
        for (var i = 0; i < str.length; i += 2) {
            a.push(parseInt("0x" + str.substr(i, 2), 16));
        }
        return a;
    };
    NfcCardReader.prototype.createHexString = function (arr) {
        var result = "";
        var str = "";
        try {
            if (arr) {
                for (var i = 0; i < arr.length; i++) {
                    // app.log("X:" + arr[i]);
                    try {
                        str = Number(arr[i]).toString(16);
                        if (str === "NaN") {
                            this.appHandler.log("Seltsam Byte " + arr[i]);
                            str = arr[i].toString();
                        }
                        //if (isNaN(str)) {
                        //app.log("NAN:" + arr[i])
                        // str = arr[i].toString();
                        //}
                        result += (str.length === 1 ? "0" + str : str) + "";
                    }
                    catch (e) {
                        // Empty
                    }
                }
            }
        }
        catch (e) {
            this.logError(e);
        }
        return result;
    };
    NfcCardReader.prototype.clearWaitList = function () {
        var _this = this;
        // 10 Sekunden das genügt. Soll den Client nicht zu lange Blockieren.
        var maxWaitTime = new Date().getTime() - 10000;
        if (this.WaitList) {
            this.WaitList = this.WaitList.filter(function (e) {
                if (e.date < maxWaitTime) {
                    // app.log("Close nix passiert ")
                    _this.sendResponse(e.request, 0, false);
                    return false;
                }
                return true;
            });
        }
    };
    NfcCardReader.prototype.longToByteArray = function (long) {
        try {
            // we want to represent the input as a 8-bytes array
            var byteArray = [0, 0, 0, 0, 0, 0, 0, 0];
            for (var index = 0; index < byteArray.length; index++) {
                var byte = long & 0xff;
                byteArray[index] = byte;
                long = (long - byte) / 256;
            }
            return byteArray;
        }
        catch (e) {
            // empty
        }
    };
    NfcCardReader.prototype.byteArrayToLong = function (byteArray) {
        if (this.isNewMode) {
            return this.byteArrayToLong_new(byteArray);
        }
        else {
            return this.byteArrayToLong_old(byteArray);
        }
    };
    NfcCardReader.prototype.byteArrayToLong_new = function (byteArray) {
        var value2 = parseInt(this.createHexString(byteArray), 16);
        return value2;
    };
    NfcCardReader.prototype.byteArrayToLong_old = function (byteArray) {
        var value = 0;
        try {
            for (var i = byteArray.length - 1; i >= 0; i--) {
                value = (value * 256) + byteArray[i];
            }
        }
        catch (e) {
            // empty
        }
        return value;
    };
    return NfcCardReader;
}());
exports.NfcCardReader = NfcCardReader;
