"use strict";
exports.__esModule = true;
var Process = require("child_process");
var Os = require("os");
var path = require("path");
var fs = require("fs");
var ScreenModeConfig = /** @class */ (function () {
    function ScreenModeConfig() {
    }
    return ScreenModeConfig;
}());
exports.ScreenModeConfig = ScreenModeConfig;
/** OS Relavet Tasks  */
var OsTools = /** @class */ (function () {
    function OsTools() {
        this.isLinux = this.isWindows = false;
        this.xinputProg = "xinput";
        // In Windows speichern wir die da wo die app ist
        this.configFile = "DikasServiceApp.json";
        if (Os.platform().toString().indexOf("win") > -1) {
            this.isWindows = true;
            this.xinputProg = "node";
            this.configFile = path.join(__dirname, "..", this.configFile);
        }
        else {
            this.isLinux = true;
            this.configFile = OsTools.getHomePath(this.configFile);
        }
        this.initBrowserName();
    }
    /**
     * Liefert den Pfad in dem die Conifgs gespeichert werden.
     *
     * @static
     * @param {string} elm Dateiname
     * @returns {string}
     *
     * @memberOf OsTools
     */
    OsTools.getHomePath = function (elm) {
        var startDir = __dirname;
        if (Os.homedir) {
            startDir = Os.homedir();
        }
        return path.normalize(path.join(startDir, elm));
    };
    OsTools.prototype.initBrowserName = function () {
        if (this.isLinux) {
            // Ubuntu
            if (fs.existsSync("/usr/bin/chromium-browser")) {
                this.browserName = "chromium";
            }
            else {
                // Debian wheezry ggf auch anders
                this.browserName = "chromium";
            }
        }
        else {
            this.browserName = "chrome";
        }
    };
    OsTools.getTempFileName = function () {
        return path.normalize(path.join(Os.tmpdir(), new Date().getTime() + ".tmp"));
    };
    /** Setzt den Screen Mode auf den neuen Modus */
    OsTools.prototype.setScreenMode = function (config, cb) {
        var _this = this;
        if (config.TargetId) {
            var msg_1 = "";
            if (!config.InversionA) {
                config.InversionA = 0;
            }
            if (!config.InversionB) {
                config.InversionB = 0;
            }
            if (!config.Swap) {
                config.Swap = 0;
            }
            // xinput set-prop 12 "Evdev Axis Inversion" 1, 1 -> ohne " übergeben
            var args_1 = ["set-prop", config.TargetId.toString(), "Evdev Axis Inversion",
                config.InversionA.toString(), config.InversionB.toString()];
            if (this.isWindows) {
                args_1.unshift("--version");
            }
            this.runProg(this.xinputProg, args_1, function (data) {
                // console.log("Axis " + data);
                msg_1 = data;
                args_1 = ["set-prop", config.TargetId.toString(), "Evdev Axes Swap", config.Swap.toString()];
                // Pacth für die Version
                if (_this.isWindows) {
                    args_1.unshift("--version");
                }
                _this.runProg(_this.xinputProg, args_1, function (data) {
                    // console.log("Axes " + data);
                    msg_1 += "\r\n" + data;
                    cb(msg_1);
                }, function (err) {
                    msg_1 += "Error :" + err;
                    cb(msg_1);
                });
            }, function (err) {
                msg_1 += "Error :" + err;
                cb(msg_1);
            });
        }
        else {
            cb("Error keine TargetId");
        }
    };
    OsTools.getIP = function () {
        var s = [];
        var tag;
        var l = Os.networkInterfaces();
        for (var elm in l) {
            if (elm.substr(0, 2) !== "lo") {
                tag = l[elm];
                for (var i = 0; i < tag.length; i++) {
                    if (tag[i].family === "IPv4") {
                        s.push({ Interface: elm, IP: tag[i].address });
                    }
                }
            }
        }
        return s;
    };
    OsTools.prototype.powerOff = function () {
        var _this = this;
        var exec = require("child_process").exec;
        if (this.isLinux) {
            // Browser und dann System Stromlos machen
            this.runCmd("pkill", ["-3", this.browserName], function (msg) {
                _this.runCmd("poweroff", [], function (msg) {
                    // empty
                });
            });
            // exec("sudo /sbin/poweroff");
        }
        if (this.isWindows) {
            exec("shutdown -s -t 0");
        }
    };
    OsTools.prototype.switchMode = function () {
        var ks = require("node-key-sender");
        ks.sendCombination(["alt", "tab"]);
    };
    /** führt die Befehle mit Sudo aus  */
    OsTools.prototype.runCmd = function (cmd, extra, cb) {
        // Fuehrt den den Fehl mit sudo aus
        var msg = { IsOk: false, Message: "" };
        if (extra) {
            extra.unshift(cmd);
        }
        else {
            extra = [cmd];
        }
        if (this.isLinux) {
            this.runProg("sudo", extra, function (data) {
                msg.IsOk = true;
                msg.Message = data + "\n";
                cb(msg);
            }, function (err) {
                msg.Message = err;
                cb(msg);
            });
        }
        else {
            msg.Message = "Error OS:Windows";
            cb(msg);
        }
    };
    OsTools.prototype.runCmdNoSudo = function (cmd, extra, cb) {
        // this.log("Starte " + cmd + " " + extra.join(" "));
        var proc = this.runProg(cmd, extra, function (res) {
            cb(res);
        }, function (err) {
            cb(null);
        });
    };
    OsTools.prototype.runProg = function (prog, args, done, error) {
        console.log("Run " + prog + " " + args.join(" "));
        var s = Process.spawn(prog, args);
        var data = "";
        s.stdout.on("data", function (d) {
            data += d.toString();
        });
        s.stderr.on("data", function (d) {
            console.log("RunCMD Error: " + prog + " " + args.join(" ") + " " + d);
            data += d.toString();
        });
        s.on("error", function (err) {
            console.log("Error ", err);
            if (error) {
                error(err);
            }
        });
        s.on("exit", function (code, singnal) {
            done(data);
        });
    };
    /** Liefert die Infos für den Touch Screen  */
    OsTools.prototype.getScreenInfos = function (cb) {
        var args = ["list"];
        if (this.isWindows) {
            args = ["--version"];
        }
        this.runProg(this.xinputProg, args, function (data) {
            cb(data);
        }, function (err) {
            cb(err);
        });
    };
    return OsTools;
}());
exports.OsTools = OsTools;
