"use strict";
exports.__esModule = true;
var AppModel_1 = require("./model/AppModel");
var AppHandler_1 = require("./AppHandler");
var NfcCardReader_1 = require("./NfcCardReader");
var OsTools_1 = require("./OsTools");
var DikasSchank_1 = require("./schank/DikasSchank");
var express = require("express");
var https = require("https");
var net = require("net");
var fs = require("fs");
var path = require("path");
var bodyParser = require("body-parser");
/// <reference path="./../typings/index.d.ts" />
var SerialPort = require("serialport");
var LDC = /** @class */ (function () {
    function LDC() {
        this.initPrinterCount = 0;
        this.printElements = [];
        this.syslog = "";
        this.isSystemUpdateDone = true;
        // private bonServers: BonPrinter[] = [];
        this.bonServerInfo = [];
        this.appHandler = new AppHandler_1.AppHandler();
        this.appHandler.log("StartUP IsLinux:" + this.appHandler.Os.isLinux + " Browser:" + this.appHandler.Os.browserName);
        // Starten Web Server muss vor Config passieren
        this.initExpressApp();
        // So nun noch die dienste Starten
        this.runConfig(this.appHandler.Config);
        this.lastServerState = new AppModel_1.ServerState();
        this.printElements = [];
        // Hack damit wir nur 1 mal den express übergeben alter blöder code :-(
        // this.app["log"] = this.appHandler.log;
        // this.app["logError"] = this.appHandler.logError;
        // this.app["$Config"] = this.appHandler.Config;
        // this.app["$isLinux"] = this.appHandler.Os.isLinux;
        // Starten des Web Servers
        this.initHttpServer();
        // Initallisieren den CouchProy
        // let couchProxy = new CouchProxy(this.appHandler);
    }
    LDC.prototype.extractID = function (s) {
        // /dev/ttyUSB0
        // /dev/ttyS14
        var id = 0;
        var pos = s.indexOf("ttyS");
        if (pos === -1) {
            pos = s.indexOf("USB");
            if (pos > 0) {
                pos += 3;
            }
            else {
                pos = s.indexOf("COM");
                if (pos > -1) {
                    pos += 3;
                }
            }
        }
        else {
            pos += 4;
        }
        if (pos > 0) {
            id = parseInt(s.substr(pos));
            // console.log("aus " + s + " wurden " + id);
        }
        return id;
    };
    LDC.prototype.loadDefaultPrinter = function (cb) {
        var _this = this;
        var comcount = 0;
        if (this.appHandler.Os.isWindows) {
            for (var i = 1; i < 9; i++) {
                // Wir beginnen ja mir 1
                this.bonServerInfo.push({ Name: "COM" + i, "Port": 9104 + i, USBName: null, IsOk: true, Server: null });
            }
            cb();
        }
        else {
            SerialPort.list(function (err, ports) {
                if (ports) {
                    var targetPort = [];
                    var ttys = ports.filter(function (d) { return d.comName.indexOf("ttyS") > -1; });
                    ttys.sort(function (a, b) { return _this.extractID(a.comName) - _this.extractID(b.comName); });
                    var usbs = ports.filter(function (d) { return d.comName.indexOf("USB") > -1; });
                    usbs.sort(function (a, b) { return _this.extractID(a.comName) - _this.extractID(b.comName); });
                    if (usbs.length > 0) {
                        if (ttys.length > 7) {
                            ttys = ttys.slice(0, 7);
                        }
                        targetPort = ttys.concat(usbs);
                    }
                    else {
                        targetPort = ttys;
                    }
                    targetPort.forEach(function (port) {
                        // Wir machen die ersten 8 Comports auf
                        if (comcount < 8) {
                            _this.bonServerInfo.push({ Name: port.comName, "Port": (9105 + comcount), USBName: null, IsOk: true, Server: null });
                            comcount++;
                        }
                    });
                }
                cb();
            });
        }
    };
    // private https: any;
    LDC.prototype.initHttpServer = function () {
        var _this = this;
        // Starten auf http
        var server = this.appHandler.App.listen(3000, function () {
            _this.appHandler.verbose("Express server listening on port " + 3000);
        });
        var keyFile = path.resolve(__dirname, "..", "dikas.key");
        if (fs.existsSync(keyFile)) {
            var privateKey = fs.readFileSync(keyFile);
            var certificate = fs.readFileSync(path.resolve(__dirname, "..", "dikas.crt"));
            this.appHandler.log("SSL Server listening on port 3001");
            var http2 = https.createServer({
                key: privateKey,
                cert: certificate
            }, this.appHandler.App).listen(3001);
        }
        else {
            this.appHandler.logError("Keine SSL Cert gefunden " + keyFile);
        }
    };
    LDC.prototype.initExpressApp = function () {
        var _this = this;
        this.appHandler.App = express();
        this.appHandler.App.use(express.static(path.join(__dirname, "public")));
        this.appHandler.App.use(bodyParser.json());
        this.appHandler.App.use(function (req, res, next) {
            res.header("Access-Control-Allow-Origin", "*");
            res.header("Access-Control-Allow-Methods", "GET,POST");
            res.header("Access-Control-Allow-Headers", "x-pass,content-type");
            next();
        });
        // this.appHandler.AppPub = express();
        // this.appHandler.AppPub.use(bodyParser.json());
        // this.appHandler.AppPub.use((req, res, next) => {
        //     res.header("Access-Control-Allow-Origin", "*");
        //     res.header("Access-Control-Allow-Methods", "GET,POST");
        //     res.header("Access-Control-Allow-Headers", "x-pass,content-type");
        //     next();
        // });
        // Ok wir liefern einfach mal die Standard Page aus.
        this.appHandler.App.get("/", function (req, res) {
            res.redirect("index.html");
        });
        /** Senden der XInput Info */
        this.appHandler.App.get("/rest/display", function (req, res) {
            _this.appHandler.Os.getScreenInfos(function (info) {
                res.send({ IsOk: true, Message: info, Config: _this.appHandler.Config.Screen });
            });
        });
        this.appHandler.App.post("/rest/display", function (req, res) {
            var t = req.body;
            if (t && t.TargetId) {
                // Never Trust User Input!
                _this.appHandler.Config.Screen = new OsTools_1.ScreenModeConfig();
                _this.appHandler.Config.Screen.InversionA = t.InversionA;
                _this.appHandler.Config.Screen.InversionB = t.InversionB;
                _this.appHandler.Config.Screen.Swap = t.Swap;
                _this.appHandler.Config.Screen.TargetId = t.TargetId;
                _this.appHandler.saveConfig();
                _this.appHandler.log("Set Screen " + JSON.stringify(_this.appHandler.Config.Screen));
                _this.appHandler.Os.setScreenMode(_this.appHandler.Config.Screen, function (a) {
                    res.send({ IsOk: true, Message: a });
                });
            }
            else {
                res.send({ IsOk: false, Message: "keine TargetId" });
            }
        });
        this.appHandler.App.get("/rest/sysinfo", function (req, res) {
            var result = {
                IP: OsTools_1.OsTools.getIP()
            };
            // 1. Checken ob wir den Drucker haben.
            result.BonPrinter = []; // this.bonServerInfo;
            result.OS = process.platform;
            result.Config = _this.appHandler.Config;
            // Wir geben eine Version mit dann können wir entscheiden ob wir die Funktion besitzen
            result.Version = 1;
            if (!_this.lastServerState.CheckDate || _this.lastServerState.CheckDate < new Date()) {
                _this.updateServiceStatus();
            }
            else {
                result.Service = _this.lastServerState;
            }
            if (_this.nfcHandler) {
                _this.nfcHandler.haveReader(function (haveReader) {
                    result.CardReader = haveReader;
                    res.json(result).end();
                });
            }
            else {
                result.CardReader = false;
                res.json(result).end();
            }
        });
        this.appHandler.App.get("/calib", function (req, res) {
            try {
                if (_this.appHandler.Os.isLinux) {
                    _this.appHandler.log(("Calib Service write File"));
                    // Browser Beenden
                    fs.writeFileSync("/home/dikas/touch.run", "yeah");
                    // Pkill -3 damit wir sauber beendet werden
                    _this.appHandler.Os.runCmd("pkill", ["-3", _this.appHandler.Os.browserName], function (msg) {
                        _this.appHandler.verbose(("Calib Service " + JSON.stringify(msg)));
                        res.send(msg);
                    });
                }
                else {
                    res.send({ IsOk: false, Message: "Windows kein Calib" });
                }
            }
            catch (e) {
                _this.appHandler.logError("Calib Service Error " + e);
                res.send({ IsOk: false, Message: e });
            }
        });
        // Funktion fuer die IP
        this.appHandler.App.get("/rest/ip", function (req, res) {
            // log("ip");
            res.send(OsTools_1.OsTools.getIP());
        });
        this.appHandler.App.get("/rest/power", function (req, res) {
            _this.appHandler.log("System PowerOff");
            res.end();
            _this.appHandler.Os.powerOff();
        });
        this.appHandler.App.get("/rest/switch", function (req, res) {
            _this.appHandler.log("System System");
            res.end();
            try {
                _this.appHandler.Os.switchMode();
            }
            catch (e) {
                _this.appHandler.logError(e);
            }
        });
        // this.appHandler.App.get("/rest/reboot", (req, res) => {
        //     this.appHandler.log("System PowerOff");
        //     res.end();
        //     this.runCmd("shutdown", ["-hr", "0"], msg => {
        //         // empty Block is ok
        //     });
        // });
        /** System Reset ggf hängt sich das hier komplett auf */
        // this.appHandler.App.get("/rest/restart", (req, res) => {
        //     if (this.appHandler.Os.isLinux) {
        //         this.appHandler.log("Restart System");
        //         res.send({ IsOk: true });
        //         // OK wir beenden das nach der Zeit
        //         setTimeout(() => {
        //             process.exit();
        //         }, 100);
        //     } else {
        //         this.appHandler.log("Restart System kein Linux");
        //         res.send({ IsOk: false, Message: "Kein Linux" });
        //     }
        // });
        this.appHandler.App.get("/rest/vnc", function (req, res) {
            _this.appHandler.log("Restart VNC");
            if (_this.appHandler.Os.isLinux) {
                _this.appHandler.Os.runCmdNoSudo("pkill", ["x11vnc"], function () {
                    _this.appHandler.Os.runCmdNoSudo("x11vnc", ["-nopw", "-forever"], function () {
                        // Ok all done
                    });
                    res.send({ IsOk: true });
                });
            }
            else {
                res.send({ IsOk: false, Message: "Kein Linux" });
            }
        });
        this.appHandler.App.post("/rest/upconfig", function (req, res) { return _this.handleConfigUpdate(req, res); });
        this.appHandler.App.post("/rest/sslhook", function (req, res) {
            _this.handleSSL(req.body, 1);
            res.end();
        });
        this.appHandler.App.post("/rest/ssl", function (req, res) {
            try {
                // Sicherstellen das uns das auch egal ist
                process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
                var b = req.body;
                if (b != null) {
                    var c = JSON.stringify(b);
                }
                var respObj_1 = {
                    statusCode: 0,
                    body: ""
                };
                // Erst hier initallisieren da ggf die alten Clients kein needle haben
                if (!_this.needle) {
                    try {
                        _this.needle = require("needle");
                    }
                    catch (e) {
                        _this.appHandler.logError("No Needle " + e);
                        respObj_1.statusCode = 500;
                        respObj_1.body = "no needle";
                    }
                }
                // Kann sein das wir das nicht initallisieren könnten
                if (_this.needle) {
                    _this.appHandler.log("Run SSL " + req.body.url);
                    // Wir senden das wenn wir gateway timeout bekommen dann schicken wir das
                    _this.needle.request(req.body.action, req.body.url, req.body.data, { json: true }, function (err, result) {
                        if (err) {
                            _this.appHandler.log("Response Error " + req.body.url + err);
                        }
                        respObj_1.body = JSON.stringify(result.body);
                        respObj_1.statusCode = res.statusCode;
                        res.json(respObj_1);
                    });
                }
            }
            catch (e) {
                _this.appHandler.logError("SendSSL " + e);
                if (res) {
                    res.end();
                }
            }
        });
        /*
        this.appHandler.App.get("/rest/networkconfig", (req, res) => {
            this.appHandler.log("netconfig");
            this.readNetworkConfig((data) => {
                res.send({ IsOk: data != null, Config: data });
            });
        });

        this.appHandler.App.post("/rest/networkconfig", (req, res) => {
            this.appHandler.log("netconfig saved");
            let config = req.body;
            if (config.DNS && config.Interfaces) {
                setTimeout(() => {
                    this.donetwork("ifdown", config.Interfaces, () => {
                        this.writeConfig(config);
                        this.runCmd("/sbin/updatenetwork.sh", [], {}, () => {
                            this.donetwork("ifup", config.Interfaces, () => {
                                try {
                                } catch (e)
                                { }
                            });
                        });
                    });
                }, 1000);
                res.send({ IsOk: true });
            } else {
                res.send({ IsOk: false });
            }
        });
        */
        //this.appHandler.App.get("/rest/systemupdate/update", (req, res) => {
        //    // log("systemupdate/up");
        //    // sendData(res);
        //});
        this.appHandler.App.get("/rest/serverstatus", function (req, res) {
            _this.appHandler.log("serverstatus");
            var cmd = "";
            if (req.query) {
                cmd = req.query.q.toLowerCase();
            }
            var resData = { IsOk: true, Message: "", NoLog: false };
            if (_this.appHandler.Os.isLinux) {
                if (cmd === "vpnstart") {
                    // Solange wir noch kein VPN haben
                    if (!_this.lastServerState.VPN) {
                        _this.appHandler.Os.runCmd("service", ["openvpn", "start", "quickSupport"], function (msg) {
                            _this.lastServerState.VPN = msg.IsOk;
                            res.send(msg);
                        });
                    }
                }
                else if (cmd === "vpnstop") {
                    _this.appHandler.Os.runCmd("service", ["openvpn", "stop", "quickSupport"], function (msg) {
                        _this.lastServerState.VPN = false;
                        res.send(msg);
                    });
                }
                else {
                    res.send({ IsOk: false, Message: "Unbekannt" });
                }
            }
            else {
                res.send(resData);
            }
        });
        //this.appHandler.App.get("/terminal", (req, res) => {
        //    this.appHandler.log("Starte Termin xterm ");
        //    this.startProg("xterm");
        //    res.json({ Result: "Started" });
        //});
        this.appHandler.App.get("/touch", function (req, res) {
            if (_this.appHandler.Os.isLinux) {
                _this.appHandler.log("Starte Touch Calibration xterm ");
                _this.appHandler.Os.runProg("xinput_calibrator", [], function (m) {
                    // Ok alles ausgeführt
                });
                res.json({
                    Result: "Started"
                });
            }
        });
        this.appHandler.App.get("/rest/resetprinter", function (req, res) {
            _this.initPrinter(function (data) {
                res.json({ Message: data, IsOk: true });
            });
        });
        this.appHandler.App.get("/rest/testprinter/:port", function (req, res) {
            _this.appHandler.log("TestPrint");
            var s = "";
            for (var i = 0; i < 10; i++) {
                s += "Dikas Test Print\r\n";
            }
            _this.printElements.push({ Printer: req.params.port, Msg: s, ErrorCount: 0 });
            // printData();
        });
        // Logt einfach nur was hier reinkommt.
        this.appHandler.App.post("/lieferdemo", function (req, res) {
            var msg = null;
            if (req.body.text) {
                msg = req.body.message;
            }
            else {
                if (_this.isString(req.body)) {
                    msg = req.body;
                }
                else {
                    msg = JSON.stringify(req.body);
                }
            }
            if (req.body.message) {
                msg = req.body.message;
            }
            if (req.body.body) {
                msg = req.body.body;
            }
            if (msg) {
                _this.appHandler.log("Testdelivery " + msg);
            }
            else {
                res.status(500).end();
            }
        });
        // this.appHandler.App.post("/pdf/fill", (req, res) => {
        //     let pf: PdfFiller = new PdfFiller();
        //     if (req.body.convmap) {
        //         pf.fillFormFilter(req.body.file, req.body.data, req.body.convmap, (data) => {
        //             res.sendJson(data);
        //         });
        //     } else {
        //         pf.fillForm(req.body.file, req.body.data, (data) => {
        //             res.sendJson(data);
        //         });
        //     }
        // });
        // this.appHandler.App.post("/pdf/fields", (req, res) => {
        //     let pf: PdfFiller = new PdfFiller();
        //     pf.getFormValues(req.body.file, (data) => {
        //         res.sendJson(data);
        //     });
        // });
    };
    /** Über den Post initallisieren wir den Kartenleser neu  */
    LDC.prototype.handleConfigUpdate = function (req, res) {
        if (req.body) {
            //if (this.appHandler.Config.NfcPort !== req.body.NfcPort) {
            //    this.appHandler.log("Config Update" + JSON.stringify(req.body));
            //} else {
            //    this.appHandler.log("No Config Update" + JSON.stringify(req.body));
            //}
            this.runConfig(req.body);
            this.appHandler.saveConfig();
            res.send({ IsOk: true });
            res.end();
        }
    };
    /** Wird beim Laden oder aktuallisieren der Config ausgeführt */
    LDC.prototype.runConfig = function (config) {
        var _this = this;
        // Übernehmen der Config dann auch zum speichern 
        this.appHandler.Config = config;
        if (this.nfcHandler) {
            this.nfcHandler.doDestroy();
        }
        if (this.appHandler.Config.Nfc) {
            this.appHandler.log("Init NfcHandler");
            this.nfcHandler = new NfcCardReader_1.NfcCardReader(this.appHandler);
        }
        if (this.appHandler.Config.Schank) {
            if (!this.schankHandler) {
                try {
                    // Ok wir machen die einzelnen ports bis 3013 auf
                    this.appHandler.log("Init Schankhandler 1 3010");
                    this.schankHandler = new DikasSchank_1.DikasSchank(3010);
                    this.appHandler.log("Init Schankhandler 2 3011");
                    this.schankHandler = new DikasSchank_1.DikasSchank(3011);
                    this.appHandler.log("Init Schankhandler 3 3012");
                    this.schankHandler = new DikasSchank_1.DikasSchank(3012);
                    this.appHandler.log("Init Schankhandler 4 3013");
                    this.schankHandler = new DikasSchank_1.DikasSchank(3013);
                }
                catch (e) {
                    this.appHandler.log("start Schank error " + e);
                }
            }
        }
        if (this.appHandler.Config.Printer) {
            if (!this.printHandler) {
                this.appHandler.log("Init Bonprinter");
                // Alle 200 ms drucken wir aus dann kann es da auch keinen Stau geben.
                this.printHandler = setInterval(function () { _this.writeToPrinter(); }, 200);
                // Ok wir laden erst mal alles aus dem Serial Port raus.
                this.loadDefaultPrinter(function () {
                    _this.initPrinter();
                });
            }
        }
        // So nun führen wir gleich noch ein paar Sachen aus.
        if (this.appHandler.Config.Screen && this.appHandler.Config.Screen.TargetId) {
            new OsTools_1.OsTools().setScreenMode(this.appHandler.Config.Screen, function (res) {
                _this.appHandler.log("ScreenConfig " + res);
            });
        }
    };
    LDC.prototype.isString = function (o) {
        return (Object.prototype.toString.call(o) === "[object String]");
    };
    LDC.prototype.handleSSL = function (data, count) {
        var _this = this;
        if (!count) {
            count = 0;
        }
        try {
            var respObj = {
                statusCode: 0,
                body: ""
            };
            // Erst hier initallisieren da ggf die alten Clients kein needle haben
            if (!this.needle) {
                try {
                    this.needle = require("needle");
                }
                catch (e) {
                    this.appHandler.logError("No Needle " + e);
                    respObj.statusCode = 500;
                    respObj.body = "no needle";
                }
            }
            // Kann sein das wir das nicht initallisieren könnten
            if (this.needle) {
                this.appHandler.log("Run " + data.url);
                // Wir senden das wenn wir gateway timeout bekommen dann schicken wir das
                this.needle.request(data.action, data.url, data.data, { json: true }, function (err, res) {
                    if (err) {
                        _this.appHandler.log("Response Error " + data.url + err);
                    }
                    // Wenn wir keine Antwort bekommen haben oder einen Gateway Timeout dann schicken wir das innerhalb von 5 Sekunden noch mal
                    if (!res.statusCode || res.statusCode === 503 || res.statusCode === 502) {
                        if (count < 5) {
                            _this.appHandler.log("Response NOK " + res.statusCode + " " + data.url + " Retry " + count);
                            // Ok fehler wollen wir bis zu 5 mal versuchen.
                            setTimeout(function () {
                                _this.handleSSL(data, count + 1);
                            }, 5000);
                        }
                        else {
                            _this.appHandler.log("Response NOK " + res.statusCode + " " + data.url + " Abort");
                        }
                    }
                    else {
                        _this.appHandler.log("Response ok " + res.statusCode + " " + data.url + " " + res.body);
                    }
                });
            }
        }
        catch (e) {
            this.appHandler.logError(e);
        }
    };
    LDC.prototype.updateServiceStatus = function () {
        this.lastServerState.CheckDate = new Date(new Date().getTime() + 1000 * 30);
        this.lastServerState.Web = false;
        this.lastServerState.DB = false;
        this.lastServerState.VPN = false;
    };
    LDC.prototype.sendData = function (res, force) {
        var _this = this;
        // Wenn Systemupdate durch ist auf jeden fall beenden
        force = force || this.isSystemUpdateDone;
        if (this.syslog.length > 0 || force) {
            try {
                res.send({ IsOk: this.isSystemUpdateDone, Message: this.syslog });
            }
            catch (e) {
                // Empty ok geht halt nicht mehr.
            }
            this.syslog = "";
        }
        else {
            if (!res.Count) {
                res.Count = 0;
            }
            if (res.Count < 50) {
                setTimeout(function () {
                    res.Count++;
                    _this.sendData(res, false);
                }, 1000);
            }
            else {
                this.sendData(res, true);
            }
        }
    };
    // private poweroff() {
    //     this.appHandler.Os.powerOff();
    // }
    LDC.prototype.initPrintServer = function (config) {
        var _this = this;
        this.appHandler.log("Init Bonprinter " + config.Port + " " + config.Name);
        try {
            config.Server = net.createServer(function (c) {
                var curElm = [];
                c.on("error", function (e) {
                    _this.appHandler.logError("ErrorPrintSrv" + e + " " + config.Name);
                });
                c.on("end", function () {
                    _this.appHandler.verbose("Print BON reviced");
                    _this.printElements.push({ Printer: config, Msg: curElm.slice(0), ErrorCount: 0 });
                    curElm.length = 0;
                });
                c.on("data", function (data) {
                    // s += data;
                    for (var i = 0; i < data.length; i++) {
                        curElm.push(data[i]);
                    }
                });
            });
            // Starten TCP Server
            config.Server.listen(config.Port, function () {
                // Ok und nun?
            });
        }
        catch (e) {
            this.appHandler.logError("Error " + e);
        }
    };
    LDC.prototype.initPrinter = function (cb) {
        var _this = this;
        var elm;
        var p = "/dev/usb/";
        var printer;
        var config;
        for (var i = 0, im = this.bonServerInfo.length; i < im; i++) {
            if (this.bonServerInfo[i].USBName) {
                this.bonServerInfo[i].IsOk = false;
            }
        }
        // Nun alle drucker finden und anpassen
        if (fs.existsSync(p)) {
            this.appHandler.verbose("Check printers in " + p);
            printer = fs.readdirSync(p);
            if (printer) {
                this.appHandler.verbose("Check printers in " + JSON.stringify(printer));
                printer.forEach(function (elm) {
                    _this.appHandler.verbose("check is printer " + elm);
                    if (elm.indexOf("lp") > -1) {
                        var targets = _this.bonServerInfo.filter(function (d) { return d.USBName === elm; });
                        if (targets.length === 0) {
                            _this.appHandler.verbose("found USB Printer " + elm);
                            _this.bonServerInfo.push({ Name: p + elm, USBName: elm, Port: 0, IsOk: true, Server: null });
                        }
                        else {
                            targets[0].IsOk = true;
                        }
                    }
                    else {
                        _this.appHandler.verbose("not a printer " + elm);
                    }
                });
            }
        }
        else {
            this.appHandler.verbose("no usb printers");
        }
        p = "/dev";
        if (fs.existsSync(p)) {
            // ok wir haben hin und wieder noch diese Drucker der mit /usb/lp0 nicht geht aber mit dem ACM0
            printer = fs.readdirSync(p);
            if (printer) {
                printer.forEach(function (elm) {
                    if (elm === "ttyACM0") {
                        var targets = _this.bonServerInfo.filter(function (d) { return d.USBName === elm; });
                        if (targets.length === 0) {
                            _this.appHandler.log("USB Printer " + elm);
                            _this.bonServerInfo.push({ Name: p + elm, USBName: elm, Port: 0, IsOk: true, Server: null });
                        }
                        else {
                            targets[0].IsOk = true;
                        }
                    }
                });
            }
        }
        this.bonServerInfo = this.bonServerInfo.filter(function (d) {
            if (!d.USBName || d.IsOk) {
                return true;
            }
            else {
                _this.appHandler.verbose("Close Bon Printer " + d.Port + " " + d.Name);
                try {
                    d.Server.close();
                }
                catch (e) {
                    _this.appHandler.logError("Close Server " + e);
                }
                return false;
            }
        });
        // Kann erst gemacht werden nachdem alle alten weg sind.
        this.bonServerInfo.forEach(function (config) {
            if (!config.Server) {
                if (config.Port === 0) {
                    config.Port = _this.findFreePort();
                    if (config.Port > 9104) {
                        _this.appHandler.logError("Zu viele Drucker " + config.Port);
                        // Monk f�r den Close
                        config.Server = { close: {} };
                    }
                }
                // Nur wenn der drucker neu ist initalliseren wir den auch?
                _this.initPrintServer(config);
            }
        });
    };
    LDC.prototype.findFreePort = function () {
        var usedPorts = [];
        // Alle USB Drucker die einen Post haben raussuchen
        for (var i = 0, im = this.bonServerInfo.length; i < im; i++) {
            if (this.bonServerInfo[i].USBName) {
                if (this.bonServerInfo[i].Port > 0) {
                    usedPorts.push(this.bonServerInfo[i].Port);
                }
            }
        }
        // So suchen wenn wir einen free port entdeck haben zuück damit
        for (var i = 9100; i < 9105; i++) {
            if (usedPorts.indexOf(i) === -1) {
                return i;
            }
        }
        // Wird nicht mehr verwendet
        return 9110;
    };
    // private startProg(prog: string, filename?: string) {
    //     this.appHandler.log("Starte " + prog);
    //     if (!filename) {
    //         filename = "runadmin";
    //     }
    //     fs.writeFileSync("/home/dikas/" + filename, "data");
    //     // Ok danach einfach den Browser killen :-)
    //     this.runCmd("pkill", [this.appHandler.Os.browserName], (msg) => {
    //         // OK ist so.
    //     });
    // }
    LDC.prototype.checkServiceIsRunning = function (name, cb) {
        if (this.appHandler.Os.isLinux) {
            this.appHandler.Os.runCmd("service", [name, "status"], function (d) {
                if (d.IsOk) {
                    if (d.Message.length > 0) {
                        d.IsOk = d.Message.toLowerCase().indexOf("is running") > -1;
                    }
                }
                cb(d);
            });
        }
        else {
            cb({ IsOk: false, Message: "Windows" });
        }
    };
    LDC.prototype.writeToPrinter = function () {
        var _this = this;
        this.initPrinterCount++;
        // Alle 3 Sekunden USB Checken
        if (this.initPrinterCount > 12) {
            this.initPrinterCount = 0;
            this.initPrinter();
        }
        if (this.printElements.length > 0) {
            // OK als erstes suchen wir die ohne Fehler
            var data_1;
            // Zuerst das Element ohne Fehler suchen
            for (var i = 0, im = this.printElements.length; i < im; i++) {
                if (!this.printElements[i].ErrorCount) {
                    data_1 = this.printElements[i];
                    i = im;
                }
            }
            // Wenn wir nur noch fehler haben
            if (!data_1) {
                data_1 = this.printElements[0];
            }
            if (data_1.ErrorCount < 20) {
                // Drucker Suchen
                this.appHandler.log("Print to " + data_1.Printer.Name);
                if (data_1.Printer.USBName) {
                    this.printWithFile(data_1.Printer.Name, data_1, function (res) {
                        if (res) {
                            _this.removePrintJob(data_1);
                        }
                    });
                }
                else {
                    // let SerialPort = require("serialport").SerialPort;
                    var baud = 19200;
                    if (this.appHandler.Config.PrinterBaud) {
                        baud = this.appHandler.Config.PrinterBaud;
                    }
                    // OK da müssen wir ggf noch was testen wie wir welchen Port aufmachen
                    var serialPort_1 = new SerialPort.SerialPort(data_1.Printer.Name, {
                        baudrate: baud
                    });
                    // Probe, Virtual Comport Driver von Epson geht nun auch.
                    // Kann sein das wir noch Zeit brauchen zum öffnen
                    serialPort_1.on("error", function (error) {
                        _this.appHandler.log("SerialPort" + error);
                    });
                    serialPort_1.open(function (error) {
                        if (error) {
                            _this.appHandler.log("Serial Port failed to open: " + error);
                            data_1.ErrorCount++;
                        }
                        else {
                            // let msg = new Buffer(data.Msg + "1D15");
                            serialPort_1.write(data_1.Msg, function (err, results) {
                                if (err) {
                                    data_1.ErrorCount++;
                                    // this.printElements.push(data);
                                    _this.appHandler.log("Bon Printer Error" + err + " " + data_1.ErrorCount);
                                }
                                else {
                                    if (results) {
                                        _this.appHandler.log("Results print " + results);
                                    }
                                    _this.removePrintJob(data_1);
                                }
                                serialPort_1.close();
                            });
                        }
                    });
                }
            }
            else {
                this.appHandler.log("Remove Job " + data_1.ErrorCount + " " + data_1.Printer.Name);
                // So nun alle Elemente aus der Liste nehmen wir wir nicht mehr drucken können
                this.printElements = this.printElements.filter(function (e) { return e !== data_1; });
            }
        }
    };
    LDC.prototype.removePrintJob = function (a) {
        var pos = this.printElements.indexOf(a);
        if (pos > -1) {
            this.printElements.splice(pos, 1);
        }
    };
    LDC.prototype.printWithFile = function (file, data, cb) {
        var _this = this;
        var msg = new Buffer(data.Msg);
        try {
            var ws_1 = fs.createWriteStream(file, { flags: "w", encoding: null });
            if (ws_1 != null) {
                ws_1.on("error", function (err) {
                    _this.appHandler.logError("Usb " + file + " " + err);
                    data.ErrorCount++;
                    cb(false);
                });
                // So nachricht drucken
                ws_1.write(msg, null, function () {
                    _this.appHandler.log("Print USB OK");
                    ws_1.close();
                    cb(true);
                });
            }
            else {
                this.appHandler.logError("Cant read stream " + data.Printer);
                data.ErrorCount++;
            }
        }
        catch (e) {
            this.appHandler.logError("Print ERROR " + e);
            cb(false);
        }
    };
    return LDC;
}());
exports.LDC = LDC;
