"use strict";
exports.__esModule = true;
var DikasServerHandler_1 = require("./DikasServerHandler");
var SchankConfig_1 = require("./model/SchankConfig");
var SchankStatus_1 = require("./model/SchankStatus");
var ComPrintHandler_1 = require("./ComPrintHandler");
var Hogatron_1 = require("./Hogatron");
var Addimat_1 = require("./Addimat");
var OsTools_1 = require("../OsTools");
var path = require("path");
var DikasSchank = (function () {
    function DikasSchank(serverPort) {
        this.serverPort = serverPort;
        this.maxLogQ = 10;
        this.IsDebug = false;
        this.timerCount = 5;
        this.Status = new SchankStatus_1.SchankStatus();
        this.log("Dikas Schankhandler");
        this.loadConfig();
        this.initServer(serverPort);
        this.setupConfig();
        this.SrvHandler = new DikasServerHandler_1.DikasServerHandler(this);
    }
    DikasSchank.prototype.initServer = function (HttpPort) {
        var _this = this;
        if (HttpPort === void 0) { HttpPort = 3010; }
        var http = require("http");
        var express = require("express");
        this.app = express();
        var bodyParser = require("body-parser");
        this.app.use(bodyParser.json({ type: "application/json" }));
        var allowCrossDomain = function (req, res, next) {
            res.header("Access-Control-Allow-Origin", "*");
            res.header("Access-Control-Allow-Methods", "GET,PUT,POST,DELETE");
            res.header("Access-Control-Allow-Headers", "x-pass");
            next();
        };
        if (this.IsDebug) {
            this.log("ACHTUNG: IST IM DEBUG MODUS!");
        }
        this.app.use(allowCrossDomain);
        this.app.use(express.static("web"));
        http.createServer(this.app).listen(HttpPort, function () {
            _this.log("Dikas Schank auf Port " + HttpPort);
        });
        this.initWebRouting();
    };
    DikasSchank.prototype.initWebRouting = function () {
        var _this = this;
        this.app.get("/", function (req, res) {
            var newdir = path.normalize(path.join(__dirname, "..", "index.html"));
            res.sendFile(newdir);
            res.end();
        });
        this.app.post("/schank", function (req, res) {
            var data;
            if (req.body.Data) {
                data = req.body.Data;
            }
            else {
                data = req.body;
            }
            if (data) {
                _this.verbose("recv: Dikas " + JSON.stringify(data));
                _this.SchankHandler.addUmsatz(data);
                if (_this.Config.PrinterPort && req.body.PrintData && req.body.PrintData.length > 0) {
                    _this.verbose("Print Bon " + _this.Config.PrinterPort);
                    new ComPrintHandler_1.ComPrintHandler().print(_this.Config.PrinterPort, req.body.PrintData);
                }
                res.json({ IsOk: true });
            }
            else {
                res.json({ IsOk: !data });
            }
        });
        this.app.get("/schank/:kellner/:plu/:anzahl", function (req, res) {
            var bok = new SchankStatus_1.PLUBooking();
            bok.Kellner = req.params.kellner;
            bok.PLU = req.params.plu;
            bok.Count = req.params.anzahl;
            if (bok.PLU !== 0) {
                _this.SchankHandler.addUmsatz([bok]);
                res.json({ IsOk: true, Kellner: bok.Kellner, PLU: bok.PLU, Anzahl: bok.Count });
            }
        });
        this.app.get("/checkpw/:pw", function (req, res) {
            if (_this.Config.SystemPasswort && _this.Config.SystemPasswort.length > 0) {
                if (_this.Config.SystemPasswort === req.params.pw) {
                    _this.verbose("Passwort Check mit OK");
                    res.json({ IsOk: true, Key: _this.getUpdateKey() });
                }
                else {
                    _this.log("Passwort Check: Fehler mit " + req.params.pw);
                    res.json({ IsOk: false });
                }
            }
            else {
                _this.verbose("Passwort Check mit OK");
                res.json({ IsOk: true, Key: _this.getUpdateKey() });
            }
        });
        this.app.get("/status", function (req, res) {
            res.send(_this.Status);
        });
        this.app.post("/testdikasserver", function (req, res) {
            if (req.body) {
                var tx = req.body;
                _this.SrvHandler.testServer(tx.DikasUrl, function (data) {
                    res.json({ IsOk: data });
                });
            }
        });
        this.app.post("/testprinter", function (req, res) {
            if (req.body) {
                var tx = req.body;
                new ComPrintHandler_1.ComPrintHandler().print(tx.PrinterPort, "Hallo Dikas Schank\n\n\n\n");
            }
            res.json({ IsOk: true });
        });
        this.app.get("/config", function (req, res) {
            res.json(_this.Config);
        });
        this.app.post("/update/:key", function (req, res) {
            if (!_this.Config.SystemPasswort ||
                _this.Config.SystemPasswort.length === 0
                || req.params.key === _this.getUpdateKey() || req.params.key === "sys") {
                if (req.body) {
                    _this.log("Update der Einstellungen");
                    try {
                        _this.updateConfig(req.body);
                        _this.setupConfig();
                        _this.saveConfig();
                    }
                    catch (ex) {
                        _this.log("Fehler " + ex);
                    }
                }
                else {
                    _this.logError("Fehler in " + JSON.stringify(req.body));
                }
                res.json({ IsOk: true });
            }
            else {
                _this.log("Update Fehler kein Body bei Update ");
                res.json({ IsOk: false });
            }
        });
    };
    DikasSchank.prototype.updateConfig = function (config) {
        if (this.Config) {
            if (!config.SystemPasswort || config.SystemPasswort.length === 0) {
                config.SystemPasswort = this.Config.SystemPasswort;
            }
        }
        this.Config = config;
        this.Status.Config = JSON.parse(JSON.stringify(this.Config));
        this.Status.Config.SystemPasswort = "";
        this.Status.NeedPassword = (this.Config.SystemPasswort && this.Config.SystemPasswort.length > 0);
    };
    DikasSchank.prototype.getUpdateKey = function () {
        if (!this.updateKey) {
            this.updateKey = new Date().getTime().toString();
        }
        return this.updateKey;
    };
    DikasSchank.prototype.logError = function (msg) {
        this.Status.ErrorLog.push(new SchankStatus_1.LogEntry(msg));
        while (this.Status.ErrorLog.length > this.maxLogQ) {
            this.Status.ErrorLog.shift();
        }
        this.logSceen("Error: " + msg);
    };
    DikasSchank.prototype.verbose = function (msg) {
        if (this.Config && this.Config.Verbose) {
            this.log(msg);
        }
    };
    DikasSchank.prototype.log = function (msg) {
        this.Status.Log.push(new SchankStatus_1.LogEntry(msg));
        var maxlog = this.maxLogQ;
        if (this.Config && this.Config.Verbose) {
            maxlog = 30;
        }
        else {
            maxlog = 15;
        }
        while (this.Status.Log.length > maxlog) {
            this.Status.Log.shift();
        }
        this.logSceen("Info: " + msg);
    };
    DikasSchank.prototype.logSceen = function (msg) {
        console.log(new Date().toLocaleTimeString() + " Schankhandler " + msg);
    };
    DikasSchank.prototype.getAllPorts = function () {
        var _this = this;
        this.Status.IPs = OsTools_1.OsTools.getIP();
        var serialPort = require("serialport");
        serialPort.list(function (err, ports) {
            _this.Status.ComPorts.length = 0;
            _this.Status.ComPorts.push(new SchankStatus_1.ComportItem("", "Keiner"));
            _this.Status.ComPorts.push(new SchankStatus_1.ComportItem("sim", "Simulator"));
            if (ports) {
                ports.forEach(function (port) {
                    _this.Status.ComPorts.push(new SchankStatus_1.ComportItem(port.comName, port.manufacturer));
                });
            }
            else {
                console.log("Kein Comport gefunden");
            }
        });
    };
    DikasSchank.prototype.loadConfig = function () {
        var c = new SchankConfig_1.SchankConfig();
        var s = this.getConfigPath();
        this.log("Load Config " + s);
        var fs = require("fs");
        if (fs.existsSync(s)) {
            s = fs.readFileSync(s);
            try {
                if (s && s.length > 0) {
                    c = JSON.parse(s);
                }
            }
            catch (ex) {
                this.log("Load Config Error " + ex + " " + s);
            }
        }
        else {
            this.log("Load Config file not found " + s);
        }
        this.updateConfig(c);
    };
    DikasSchank.prototype.getConfigPath = function () {
        if (this.serverPort !== 3010) {
            return OsTools_1.OsTools.getHomePath("DikasSchank_" + this.serverPort + ".config");
        }
        else {
            return OsTools_1.OsTools.getHomePath("DikasSchank.config");
        }
    };
    DikasSchank.prototype.saveConfig = function () {
        try {
            var s = this.getConfigPath();
            this.log("Save Config " + s);
            var fs = require("fs");
            fs.writeFileSync(s, JSON.stringify(this.Config));
        }
        catch (e) {
            this.logError("Save Config Error" + e);
        }
    };
    DikasSchank.prototype.setupConfig = function () {
        if (!this.Config.Protokoll) {
            this.Config.Protokoll = "Addimat";
        }
        if (this.SchankHandler) {
            if (this.SchankHandler.Name !== this.Config.Protokoll) {
                this.SchankHandler = null;
            }
        }
        if (!this.SchankHandler) {
            switch (this.Config.Protokoll) {
                case "Addimat":
                    this.SchankHandler = new Addimat_1.Addimat();
                    break;
                default:
                    this.SchankHandler = new Hogatron_1.Hogatron();
                    break;
            }
            this.SchankHandler.app = this;
        }
        this.log("Using " + this.SchankHandler.Name);
        this.SchankHandler.updatePortHandler();
    };
    return DikasSchank;
}());
exports.DikasSchank = DikasSchank;
//# sourceMappingURL=DikasSchank.js.map