"use strict";
exports.__esModule = true;
var SchankStatus_1 = require("./model/SchankStatus");
var DikasServerHandler = (function () {
    function DikasServerHandler(app) {
        var _this = this;
        this.app = app;
        this.openElements = new Array();
        this.isTransfer = false;
        setInterval(function () { return _this.transfer(); }, 100);
    }
    DikasServerHandler.prototype.handleData = function (t) {
        if (t.PLU > 0) {
            t.Date = new Date();
            if (this.openElements.length < 100) {
                this.app.log("New PLU:" + t.PLU + " K:" + t.Kellner + " T:" + t.Tisch + " Count:" + t.Count);
                this.openElements.push(t);
                this.app.Status.ToServerBookings.push(t);
                if (this.app.Status.ToServerBookings.length > 10) {
                    this.app.Status.ToServerBookings.splice(0, this.app.Status.ToServerBookings.length - 10);
                }
            }
            else {
                this.app.logError("Element verworfen mehr als 100 zu senden");
            }
        }
    };
    DikasServerHandler.prototype.transfer = function () {
        var _this = this;
        if (!this.isTransfer) {
            this.app.Status.OpenFromSchank = this.openElements.length;
            if (this.openElements.length > 0) {
                this.app.verbose("Send " + this.openElements.length + " Bookings to Dikas");
                this.isTransfer = true;
                var it_1 = this.openElements.pop();
                if (!it_1.$MaxCount) {
                    it_1.$MaxCount = 100;
                }
                this.sendToServer(this.app.Config.DikasUrl, it_1, function (body) {
                    it_1.$IsDone = true;
                    _this.isTransfer = false;
                    _this.app.Status.ToServer++;
                    if (body) {
                        _this.app.log("Send Dikas OK P:" + it_1.PLU + " K:" + it_1.Kellner + " T:" + it_1.Tisch + " A:" + it_1.Count);
                    }
                    else {
                        _this.app.verbose("Send Dikas Error " + JSON.stringify(body));
                    }
                }, function (err) {
                    _this.app.logError("Send Dikas Error" + err);
                    _this.app.Status.ToServerError++;
                    setTimeout(function () { return _this.isTransfer = false; }, 500);
                    if (_this.openElements.length < 100) {
                        _this.openElements.push(it_1);
                    }
                });
            }
        }
    };
    DikasServerHandler.prototype.testServer = function (serverUrl, cb) {
        var testData = new SchankStatus_1.PLUBooking();
        testData.Kellner = 0;
        testData.PLU = 0;
        testData.Tisch = 0;
        this.sendToServer(serverUrl, testData, function () {
            cb(true);
        }, function () {
            cb(false);
        });
    };
    DikasServerHandler.prototype.sendToServer = function (url, data, done, error) {
        if (this.app.Config.DikasUrl && this.app.Config.DikasUrl.length > 0) {
            try {
                var protokoll = "http";
                if (this.app.Config.DikasUrl.indexOf(".dikas") > -1) {
                    protokoll = "https";
                }
                var http = require(protokoll);
                var newUrl = protokoll + "://" + url
                    + "/rest/gastro/schank"
                    + "?p=" + data.PLU + "&k=" + data.Kellner + "&t=" + data.Tisch + "&c=1";
                this.app.verbose("URL " + newUrl);
                http.get(newUrl, function (res) {
                    var body = "";
                    res.on("data", function (chunk) {
                        body += chunk;
                    });
                    res.on("end", function () {
                        if (res.statusCode === 200) {
                            try {
                                done(JSON.parse(body));
                            }
                            catch (e) {
                                done("");
                            }
                        }
                        else {
                            error("StatusCode " + res.statusCode);
                        }
                    });
                }).on("error", function (e) {
                    error(e);
                });
            }
            catch (e) {
                error("SendToServer: " + e);
            }
        }
        else {
            error("Kein Dikas Server eingetragen");
        }
    };
    return DikasServerHandler;
}());
exports.DikasServerHandler = DikasServerHandler;
//# sourceMappingURL=DikasServerHandler.js.map