"use strict";
var ComportHandler = (function () {
    function ComportHandler(app) {
        var _this = this;
        this.app = app;
        this.SerialPort = require("serialport").SerialPort;
        this.inUse = false;
        this.reviedData = [];
        this.initSerialPort();
        this.errorCount = 0;
        setInterval(function () { return _this.checkTimeout(); }, 500);
    }
    ComportHandler.prototype.isOpen = function () {
        return this.IsOpen;
    };
    ComportHandler.prototype.close = function () {
        this.app.log("close comport");
        if (this.serialPort) {
            this.IsOpen = false;
            this.serialPort.close();
            this.serialPort = null;
        }
    };
    ComportHandler.prototype.initSerialPort = function () {
        var _this = this;
        this.errorCount = 0;
        if (this.serialPort) {
            try {
                this.serialPort.close();
            }
            catch (e) {
                this.app.log("close serialport Error " + e);
            }
        }
        if (!this.app.Config.Databits) {
            this.app.Config.Databits = 8;
        }
        if (!this.app.Config.Stopbits) {
            this.app.Config.Stopbits = 1;
        }
        if (!this.app.Config.Parity) {
            this.app.Config.Parity = "none";
        }
        this.app.log("open comport " + this.app.Config.ComDevice + " " + this.app.Config.ComBaud + " "
            + this.app.Config.Databits + "," + this.app.Config.Parity + "," + this.app.Config.Stopbits);
        this.serialPort = new this.SerialPort(this.app.Config.ComDevice, {
            baudrate: this.app.Config.ComBaud,
            stopbits: this.app.Config.Stopbits,
            databits: this.app.Config.Databits,
            parity: this.app.Config.Parity
        }, false);
        this.serialPort.on("open", function () {
            _this.app.verbose("comport open");
            _this.IsOpen = true;
            _this.inUse = false;
        });
        this.serialPort.on("data", function (data) {
            _this.lastDataDate = new Date();
            var n;
            var t1 = "";
            for (var i = 0, im = data.length; i < im; i++) {
                n = data.readInt8(i);
                _this.reviedData.push(n);
                t1 += n.toString() + " ";
            }
            _this.app.verbose("recv: " + data.toString("ascii") + " ( " + t1 + ")");
            if (_this.app.SchankHandler.handleResponse(_this.reviedData)) {
                _this.errorCount = 0;
                _this.inUse = false;
                if (_this.dataCallBack) {
                    _this.dataCallBack(_this.reviedData);
                }
                else {
                    _this.app.verbose("(End no CB):  \"" + _this.reviedData + "\"");
                }
            }
            else {
                if (_this.dataCallBack) {
                    _this.app.verbose("waiting...");
                }
                else {
                    _this.app.verbose("wating next... CB LOST");
                }
            }
        });
        this.serialPort.on("close", function (data) {
            _this.inUse = false;
            _this.app.log("comport closed");
            _this.IsOpen = false;
        });
        this.serialPort.open(function (err) {
            if (err) {
                _this.IsOpen = false;
                _this.app.logError("Error opening port: " + err.message);
            }
        });
    };
    ComportHandler.prototype.checkTimeout = function () {
        if (this.inUse) {
            if (this.dataCallBack) {
                var targetTimeout = this.lastDataDate.getTime() + this.app.Config.SendTimeOut;
                if (new Date().getTime() > targetTimeout) {
                    this.inUse = false;
                    this.errorCount++;
                    if (this.errorCount > 5) {
                        this.app.logError("ReInitComport " + this.app.Config.SendTimeOut);
                        this.initSerialPort();
                    }
                    this.dataCallBack(this.reviedData);
                }
            }
        }
    };
    ComportHandler.prototype.write = function (text, cb) {
        var _this = this;
        var t1 = "";
        for (var i = 0, im = text.length; i < im; i++) {
            t1 += text[i] + " ";
        }
        if (cb) {
            this.app.verbose("send: " + t1);
        }
        else {
            this.app.verbose("send: NoCB \"" + t1 + "\"");
        }
        this.dataCallBack = cb;
        if (!this.reviedData) {
            this.reviedData = [];
        }
        else {
            this.reviedData.length = 0;
        }
        if (!this.app.Config.SendTimeOut) {
            this.app.Config.SendTimeOut = 2000;
        }
        this.lastDataDate = new Date();
        this.inUse = true;
        this.serialPort.write(text, function (err, results) {
            if (err) {
                _this.app.logError("comport Error " + err);
                _this.inUse = false;
                if (cb) {
                    cb([]);
                }
            }
        });
    };
    ComportHandler.prototype.sendData = function (data, cb) {
        if (!this.IsOpen) {
            this.app.verbose("Error no senddata  Reinit Comport " + data);
            this.initSerialPort();
            cb(null);
        }
        else {
            this.inUse = true;
            this.write(data, function (resp) {
                if (cb) {
                    cb(resp);
                }
            });
        }
    };
    return ComportHandler;
}());
exports.ComportHandler = ComportHandler;
//# sourceMappingURL=ComportHandler.js.map